/*-*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-

    last.fm now playing plugin for ChatZilla.

    This plugin displays the last track you submitted to last.fm, and more.

    See http://chatzilla.hacksrus.com/faq/#install-script on how to install a plugin.
*/

plugin.id = "last-fm";
const __major = 1;
const __minor = '$Revision: 53 $'.match(/\d+/);
const __description = 'Display most recent last.fm track.';

const __maxArtistData = 7
const __maxArtists = 10


plugin.init =
function init(glob)
{
    /* This function will be called when chatzilla first loads the plugin.
     * We initialize some of the plugin object's properties, define a new
     * command and some new prefs, and print out a message.
     */     
    plugin.major = __major;
    plugin.minor = __minor;    
    plugin.version = __major + "." + __minor
    plugin.description = __description;

    plugin.prefs = [
        /* pref name                default value */
        ["lastfm.user",             null],
    ];
    client.prefManager.addPrefs(plugin.prefs);

    plugin.cmdlist = [
        /* basic commands */
        ["lastfm-info", cmdInfo, null, "[<text>] [<...>]"],
        ["lastfm-np", cmdLastFmNowPlaying, null, "[<cmd>] [<...>]"],
        ["lastfm-set-user", cmdLastFmSetUser, null, "<username>"],

        /* user info commands (/i* = display an info message to your eyes only) */
        ["artists", cmdLastFmArtists, null, "[<username>] [<maxno>]"],
        ["iartists", cmdLastFmArtists, null, "[<username>] [<maxno>]"],

        /* artist info commands (/i* = display an info message to your eyes only) */
        ["tags", cmdArtistData, null, "[<artist>] [<...>]"],
        ["itags", cmdArtistData, null, "[<artist>] [<...>]"],
        ["albums", cmdArtistData, null, "[<artist>] [<...>]"],
        ["ialbums", cmdArtistData, null, "[<artist>] [<...>]"],
        ["tracks", cmdArtistData, null, "[<artist>] [<...>]"],
        ["itracks", cmdArtistData, null, "[<artist>] [<...>]"],
        ["similar", cmdArtistData, null, "[<artist>] [<...>]"],
        ["isimilar", cmdArtistData, null, "[<artist>] [<...>]"],

        /* aliases */
        ["np", "lastfm-np me np:", null],
        ["inp", "lastfm-np lastfm-info You're playing", null],
    ];
    client.commandManager.defineCommands(plugin.cmdlist);

    display(
        '\x02' + plugin.id + ' plugin version ' + plugin.version + ' loaded.\x0F\n' +
        'This plugin displays the last track you submitted to last.fm, and more.' +
        '\n' +
        'Examples:\n' +
        '    /lastfm-set-user RNR\n' +
        '    /np\n' +
        '    /tags\n' +
        '    /tags the beatles\n' +
        "    /lastfm-np say I'm now playing\n"
    );

    return true;
}


plugin.enable =
function enable()
{
    /* This function is called by Chatzilla when the
       command "/enable-plugin plugin-name" is issued.
     */
    display(plugin.id + ' enabled.');
    return true;
}


plugin.disable =
function disable()
{
    /* This function is called by Chatzilla when the
       command "/disable-plugin plugin-name" is issued.
     */
    display (plugin.id + ' disabled.');
    return true;
}


function callLastFm(path, callback)
{
    var req = new XMLHttpRequest();
    if (!req) {
        display("Failed to create XMLHttpRequest object!", MT_ERROR);
        return;
    }

    /* Make an async RSS request to last.fm */
    req.open('GET', 'http://ws.audioscrobbler.com/1.0/' + path, true); 
    req.onreadystatechange = function() {
        if (req.readyState == 4) {
            if (req.status != 200) {
                display("last.fm request returned HTTP status " + req.status, MT_ERROR);
            } else {
                callback(req);
            }
        }
    }
    req.send(null);
}


function callLastFmUser(path, user, callback)
{
    if (!user) {
        user = client.prefs['lastfm.user']
        if (!user) {
            display("Use '/lastfm-set-user <username>' to set your account name first!", MT_ERROR);
            return;
        }
    }

    callLastFm('user/' + user + '/' + path, callback)
}


function say(text)
{
    dispatch('say ' + text)
}


function getChild(node, tagname)
{
    return node.getElementsByTagName(tagname).item(0).firstChild.data;
}


function cmdInfo(e)
{
    display(e.textList.join(' '))
}


function cmdLastFmNowPlaying(e)
{
    callLastFmUser('recenttracks.xml', null, function(req) {
        var cmd = 'me np:';
        if (e.cmd != undefined) {
            cmd = e.cmdList.join(" ");
        }
        var last_track = req.responseXML.getElementsByTagName('track').item(0);
        var artist = getChild(last_track, 'artist')
        var title = getChild(last_track, 'name')
        var url = getChild(last_track, 'url')
        dispatch(cmd + ' ' + artist + ' - \x02' + title + '\x0F [' + url + ']');
    })
}


function cmdLastFmSetUser(e)
{
    client.prefs['lastfm.user'] = e.username;
    display('Set last.fm user to \x02' + e.username + '\x0F');
}


function callArtistData(emit, maxno, artist, datatype)
{
    method = 'top' + datatype
    listTag = datatype.substr(0, datatype.length-1)
    switch (datatype) {
        case "tags":
            idTag = 'name'
            rankTag = 'count'
            break;
        case "albums":
        case "tracks":
            idTag = 'name'
            rankTag = 'reach'
            break;
        case "similar":
            idTag = 'name'
            rankTag = 'match'
            method = datatype
            listTag = 'artist'
            break;
        default:
            display("Dispatch for unknown datatype " + datatype, MT_ERROR);
            return;
    }

    callLastFm('artist/' + encodeURIComponent(artist) + '/' + method + '.xml', function(req) {
        var tags = req.responseXML.getElementsByTagName(listTag);

        var avg = 0;
        var elemno = 0;
        for (i = 0; i < tags.length; i++) {
            var rank = parseFloat(getChild(tags.item(i), rankTag));
            avg = avg + rank;
            if (++elemno >= maxno) break;
        }
        avg /= elemno;

        var result = []
        for (i = 0; i < elemno; i++) {
            var rank = parseFloat(getChild(tags.item(i), rankTag));
            var bold = ''
            if (rank >= avg) {
                bold = '\x02'
            }
            var text = bold + getChild(tags[i], idTag) + '\x0F(' + rank + ')';
            result.push(text);
        }

        artist = req.responseXML.firstChild.getAttribute('artist')
        emit('last.fm top ' + elemno + ' ' + datatype + ' for \x1F' + artist + '\x1F: ' + result.join(', '))
    })
}


function cmdArtistData(e)
{
    emit = say
    cmd = e.command.name
    if (cmd[0] == "i") {
        emit = display
        cmd = cmd.substr(1)
    }

    var maxno = __maxArtistData
    var artist = e.artist
    if (artist) {
        var n = parseInt(artist, 10)
        if (n) {
            maxno = n
            e.artistList.shift()
        }
        artist = e.artistList.join(' ')
    }

    if (artist) {
        callArtistData(emit, maxno, artist, cmd);
    } else {
        callLastFmUser('recenttracks.xml', null, function(req) {
            var last_track = req.responseXML.getElementsByTagName('track').item(0);
            callArtistData(emit, maxno, getChild(last_track, 'artist'), cmd);
        })
    }
}


function cmdLastFmArtists(e)
{
    emit = say;
    if (e.command.name[0] == "i") emit = display;

    var maxno = e.maxno;
    if (!maxno) {
        maxno = parseInt(e.username, 10)
        if (!maxno) {
            maxno = __maxArtists
        } else {
            e.username = null
        }
    }       

    callLastFmUser('topartists.xml', e.username, function(req) {
        var artists = req.responseXML.getElementsByTagName('artist');

        var avg = 0;
        var artistno = 0;
        for (i = 0; i < artists.length; i++) {
            var rank = parseInt(getChild(artists.item(i), 'playcount'), 10);
            avg = avg + rank;
            if (++artistno >= maxno) break;
        }
        avg /= artistno;

        var result = []
        for (i = 0; i < artistno; i++) {
            var rank = parseInt(getChild(artists.item(i), 'playcount'), 10);
            var bold = ''
            if (rank >= avg) {
                bold = '\x02'
            }
            var text = bold + getChild(artists[i], 'name') + '\x0F(' + rank + ')';
            result.push(text);
        }

        username = req.responseXML.firstChild.getAttribute('user')
        emit('Top ' + artistno + ' artists for \x1F' + username + '\x1F: ' + result.join(', '))
    })
}

